red <- paste("<font color=\"red\">")             
blue <- paste("<font color=\"blue\">") 
green <- paste("<font color=\"lightgreen\">")             
gray <- paste("<font color=\"gray\">") 
npoints <- 250
nmult <- 5   

shinyServer(function(input, output,session) {
          
    gotLibs <- reactive({
        a<-is.element("ggplot2", installed.packages()[,1])
        b<-is.element("grid", installed.packages()[,1])
        x <- c(a,b,a&b)
        names(x) <- c("ggplot2","grid","both")
        x
    })
    output$checkGraph <- renderText({
        if(input$whichgraph=="base") return("")
        if(gotLibs()[1]) library(ggplot2)
        if(gotLibs()[2]) library(grid)    
        if(!gotLibs()[3]) return(paste(red,"<h4>Warning: some R libraries not found, will use base graphics</h4>"))
        NULL       
    })
         
   data <- reactive({
        x <- seq(input$from,input$to,length=npoints)
        f<-function(x,which) {
               if(which==0) return(eval(parse(text=input$fun)))
               if(which==1) return(eval(D(parse(text=input$fun),"x")))
        }
        cbind(x,f(x,0),f(x,1)) 
   })     
   

   output$text <- renderText({
      x <- data()[,1]
      y <- data()[,2]
      yprime <- data()[,3]
      line <- paste("<h3>f(x) = ",chfun(input$fun),"</h3>")
      line[2] <- paste("<h3>f'(x) = ",chfun(deparse(D(parse(text=input$fun),"x"))),"</h3>")
      k <- nmult*input$k
      line[3] <- paste("<h3>Point for Evaluation: x =",round(x[k],2),"</h3>")
      line[4] <- paste("<h3>f(x) = ",round(y[k],3),"   f'(x) = ",round(yprime[k],3),"</h3>")
      line
   })  
         
   output$plot <- renderPlot({
        x <- data()[,1]
        y <- data()[,2]
        yprime <- data()[,3] 
        k <- nmult*input$k
        p <- c(x[k],y[k],y[k]-x[k]*yprime[k],yprime[k]) 
        if(input$whichgraph=="base" | !gotLibs()[3]) {
            par(mfrow=c(1,2))
            plot(x,y,xlab="x",ylab="",type="l",lwd=3)
            points(p[1],p[2],pch=20,cex=2)
            abline(p[3],p[4],lwd=2,col="blue")
            plot(x,yprime,xlab="x",ylab="",type="l",lwd=3)
            points(x[k],yprime[k],pch=20,cex=2,col="blue")
        }
        else {
            dta <- data.frame(x=x,y=y)
            dta1 <- data.frame(x=p[1],y=p[2])
            plt1 <- ggplot(dta, aes(x,y))+geom_line(size=1.2)+
                   geom_point(aes(x,y),data=dta1,size=4,colour="blue")+
                   geom_abline(intercept=p[3],slope=p[4],size=1.2,colour="blue")
            dta <- data.frame(x=x,y=yprime)
            dta1 <- data.frame(x=x[k],y=yprime[k])
            plt2 <- ggplot(dta, aes(x,y))+geom_line(size=1.2)+
                    geom_point(aes(x,y),data=dta1,size=4,colour="blue")    
            pushViewport(viewport(layout = grid.layout(1, 2)))
            print(plt1, vp = viewport(layout.pos.row = 1, layout.pos.col = 1))
            print(plt2, vp = viewport(layout.pos.row = 1, layout.pos.col = 2))    
        }        
   })
   
          
   output$appinfo <- renderText({ 
      HTML("<br>&nbsp;<br>&nbsp;<br>&nbsp;<br>
      <a target=\"_blank\" href=\"http://academic.uprm.edu/wrolke/myapps1.htm#deriv\"><h4>What to do</h4></a><br>
      <h6>Written by: Dr. Wolfgang Rolke<br>Created: 2/15/2015</h6>")
      
  })
  
  output$explainText <- renderText({ 
        "<h5>Explanation: This demo illustrates the Derivatives </h5>"
        
  })

})

